#include <list>

#include "epix.h"
#include "sample.h"
#include "samples.h"

using namespace ePiX;

// user-accessible list of samples to plot
samples::samples(double Hscale, double Vscale)
  : m_Hscale(Hscale), m_Vscale(Vscale) { }

samples& samples::add(const sample& s)
{
  m_elts.push_back(s);
  return *this;
}

void samples::plot(double Umin, double Umax, unsigned int num_pts) const
{
  P NE(samples::TR(Umin, Umax)); // preliminary northeast corner

  // calculate number of axis ticks
  const unsigned int Width((unsigned int)  ceil(NE.x1()/m_Hscale));
  const unsigned int Height((unsigned int) ceil(NE.x2()/m_Vscale));

  screen my_canvas(P(0,0), P(m_Hscale*Width, m_Vscale*Height));
  activate(my_canvas);

  for (std::list<sample>::const_iterator lp = m_elts.begin();
       lp != m_elts.end(); ++lp)
    lp -> plot(Umin, Umax, num_pts); // also labels the graph

  // coordinate axes
  plain(Black());
  h_axis(my_canvas.bl(), my_canvas.br(), 2*Width);
  v_axis(my_canvas.bl(), my_canvas.tl(), 2*Height);

  h_axis_labels(my_canvas.bl(), my_canvas.br(), Width,  P(0,-4), b);
  v_axis_labels(my_canvas.bl(), my_canvas.tl(), Height, P(-4,0), l);

  import(my_canvas);
  deactivate(my_canvas);
}

// return approximate TR corner of smallest screen containing our plots
P samples::TR(double Tmin, double Tmax) const
{
  double Xmax(0), Ymax(0);

  for (std::list<sample>::const_iterator lp = m_elts.begin();
       lp != m_elts.end(); ++lp)
    {
      P tmp(lp -> TR(Tmin, Tmax));

      Xmax = max(Xmax, tmp.x1());
      Ymax = max(Ymax, tmp.x2());
    }

  return P(Xmax, Ymax);
}
